/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

import ClayButton from '@clayui/button';
import ClayTable from '@clayui/table';
import PropTypes from 'prop-types';
import React, {useRef} from 'react';
import {DndProvider, createDndContext} from 'react-dnd';
import HTML5Backend from 'react-dnd-html5-backend';

import LanguageListItem from './LanguageListItem.es';
import LanguageListItemEditable from './LanguageListItemEditable.es';

const noop = () => {};

const LanguagesList = ({
	defaultLocaleId,
	locales,
	isEditable = false,
	onMakeDefault = noop,
	onEditBtnClick = noop,
	onItemDrop = noop,
	moveItem = noop,
}) => {
	const manager = useRef(createDndContext(HTML5Backend));

	return (
		<ClayTable borderless headVerticalAlignment="middle" hover={false}>
			<ClayTable.Head>
				<ClayTable.Row>
					<ClayTable.Cell expanded headingCell headingTitle>
						{Liferay.Language.get('active-language')}
					</ClayTable.Cell>

					{isEditable && (
						<ClayTable.Cell align="center">
							<ClayButton
								displayType="secondary"
								onClick={onEditBtnClick}
								small
							>
								{Liferay.Language.get('edit')}
							</ClayButton>
						</ClayTable.Cell>
					)}
				</ClayTable.Row>
			</ClayTable.Head>

			<ClayTable.Body>
				<DndProvider manager={manager.current.dragDropManager}>
					{locales.map((locale, index) => {
						const baseProps = {
							...locale,
							index,
							isDefault: defaultLocaleId === locale.localeId,
							key: locale.localeId,
						};

						return isEditable ? (
							<LanguageListItemEditable
								{...baseProps}
								isFirst={index === 0}
								isLast={index === locales.length - 1}
								onItemDrop={onItemDrop}
								onMakeDefault={onMakeDefault}
								onMoveDown={() => moveItem(index, index + 1)}
								onMoveUp={() => moveItem(index, index - 1)}
							/>
						) : (
							<LanguageListItem {...baseProps} />
						);
					})}
				</DndProvider>
			</ClayTable.Body>
		</ClayTable>
	);
};

LanguagesList.prototypes = {
	isEditable: PropTypes.bool,
	moveItem: PropTypes.func,
	onEditBtnClick: PropTypes.func,
	onItemDrop: PropTypes.func,
	onMakeDefault: PropTypes.func,
};

export default LanguagesList;
